
PROCESSOR 12F629

#include "p12f629.inc"

__CONFIG _CPD_OFF&_BODEN_OFF&_MCLRE_OFF&_PWRTE_ON&_WDT_OFF&_INTRC_OSC_NOCLKOUT

#define TXPORT GPIO,GP0

BAUD SET 9600
CONSTANT BASERAM=0x20

timeron	equ BASERAM

; serout
TXD 		equ BASERAM+1
BaudDelaytemp 	equ BASERAM+2

; bin16_2_bcd
count  		equ BASERAM+3
temp   		equ BASERAM+4
H_byte  	equ BASERAM+5
L_byte  	equ BASERAM+6
R0      	equ BASERAM+7 
R1      	equ BASERAM+8
R2      	equ BASERAM+9

; delay1ms
LOOPcnt		equ BASERAM+10

; pausems
HIcnt		equ BASERAM+11
LOcnt		equ BASERAM+12

org	0x00
goto init

org 0x04
goto int

#include "serout.inc"
#include "bin16_2_bcd.inc"
#include "bcdhelpers.inc"
#include "delay1ms.inc"
#include "delay10us.inc"
#include "pausems.inc"

init
	pausems .500

	clrf timeron 

	banksel TRISIO
	movlw b'00011100'	; pin 0 & 1 as outputs
	movwf TRISIO

	movlw b'11000000'
	movwf OPTION_REG

	banksel TMR1H
	clrf TMR1H	
	clrf TMR1L

	banksel PIE1
	movlw b'00000001'	; Enable tmr1 overflow int
	movwf PIE1	

	movlw b'11010000'	; enable peripheral amd external interrupt 
	movwf INTCON

	banksel GPIO

	bcf TXPORT
main
	nop; something
	nop
	nop
	nop
	nop
	nop
	nop
	goto main

int
	bcf T1CON, TMR1ON	; Stop the timer (exactly when the event occurred)
	btfsc PIR1, TMR1IF	; Test if timer overflow. Skip next if bit clear
	goto tmr1_overflow	
	btfss timeron, 0	; Test if timer was running	Skip next if bit set
	goto tmrl_start
tmr1_stop
	bcf timeron, 0		; clear timer indicator	

	movf TMR1H, w
	movwf H_byte
	movf TMR1L, w
	movwf L_byte	
	call bin16_2_bcd 
	
	; 10s of 1000
	bcdlow R0
	addlw 0x30
	call serout	
	call delay1ms
	
	; 1000s		
	bcdhigh R1
	addlw 0x30
	call serout	
	call delay1ms
	
	; 100s	
	bcdlow R1		
	addlw 0x30
	call serout	
	call delay1ms

	; 10s	
	bcdhigh R2
	addlw 0x30
	call serout
	call delay1ms

	; 1s	
	bcdlow R2		
	addlw 0x30
	call serout	
	call delay1ms

	movlw .10
	call serout
	call delay1ms
	movlw .13
	call serout			
	
	goto clearup

tmrl_start	
	clrf TMR1H	
	clrf TMR1L			; The offset compensates for the difference in clock cycles 	
	bsf T1CON, TMR1ON	; Start the timer 
	bsf timeron, 0		; set timer indicator
	goto clearup
	
tmr1_overflow
	bcf timeron, 0		; clear timer indicator

clearup
	bcf PIR1, TMR1IF	; clear the timer interrupt
	bcf INTCON, INTF	; clear any external interrupt that may have just happened	
	retfie

END
